@@ -52,8 +52,6 @@
 #define PORT_MQTT   1883
 #define PORT_MQTTS  8883
 
-struct curl_trc_featt;
-
 #ifdef USE_ECH
 /* CURLECH_ bits for the tls_ech option */
 # define CURLECH_DISABLE    (1 << 0)
@@ -1041,7 +1039,7 @@ struct UrlState {
                                     curl_easy_setopt(COOKIEFILE) calls */
 #endif
 
-#ifndef CURL_DISABLE_VERBOSE_STRINGS
+#ifdef CURLVERBOSE
   struct curl_trc_feat *feat; /* opt. trace feature transfer is part of */
 #endif
 
@@ -1391,7 +1389,7 @@ struct UserDefined {
 #ifdef USE_SSH
   curl_sshkeycallback ssh_keyfunc; /* key matching callback */
   void *ssh_keyfunc_userp;         /* custom pointer to callback */
-  int ssh_auth_types;    /* allowed SSH auth types */
+  uint32_t ssh_auth_types;   /* allowed SSH auth types */
   uint32_t new_directory_perms; /* when creating remote dirs */
 #endif
   uint32_t new_file_perms;      /* when creating remote files */
